@@ -102,11 +102,15 @@ sub BLI_to_NO($) {
 		$session->{appdata} = $typedata;
 
 		$session->set_info($session->{profile}) if exists($session->{profile});
+	} else {
+		# No permit info - we permit everyone
+		$session->{visibility} = VISMODE_PERMITALL;
+		$session->{groupperms} = 0xFFFFFFFF;
 	}
 
 	if(exists $bli->{5}) {
 		# Not yet implemented
-		($session->{showidle}) = unpack("N", $bli->{5}->{0}->{19719}->{data}->{0xC9});
+		($session->{showidle}) = unpack("N", $bli->{5}->{0}->{19719}->{data}->{0xC9} || pack("N", 1));
 	}
 
 	my @gids = unpack("n*", (exists($bli->{1}) and exists($bli->{1}->{0}) and exists($bli->{1}->{0}->{0}) and exists($bli->{1}->{0}->{0}->{data}->{0xC8})) ? $bli->{1}->{0}->{0}->{data}->{0xC8} : "");
@@ -161,6 +165,7 @@ sub BLI_to_NO($) {
 			$entry->{data} = $buddy->{data};
 		}
 	}
+
 	return 1;
 }
 
@@ -183,7 +188,7 @@ sub NO_to_BLI($) {
 	$vistype ||= 2;
 	$bli->{4}->{0}->{$vistype}->{data}->{0xCA} = pack("C", $session->{visibility} || VISMODE_PERMITALL);
 	$bli->{4}->{0}->{$vistype}->{data}->{0xCB} = pack("N", $session->{groupperms} || 0xFFFFFFFF);
-	$bli->{4}->{0}->{$vistype}->{data}->{0x0100} = $session->{profile} if exists($session->{profile});
+	$bli->{4}->{0}->{$vistype}->{data}->{0x0100} = $session->{profile} if $session->{profile};
 	foreach my $appdata(keys %{$session->{appdata}}) {
 		$bli->{4}->{0}->{$vistype}->{data}->{$appdata} = $session->{appdata}->{$appdata};
 	}
@@ -220,6 +225,7 @@ sub BLI_to_OSCAR($$) {
 	my $oldbli = $session->{blinternal};
 	my $oscar = $session->{bos};
 	my $modcount = 0;
+	my @snacqueue = ();
 
 	$oscar->snac_put(family => 0x13, subtype => 0x11); # Begin BL mods
 
@@ -230,6 +236,7 @@ sub BLI_to_OSCAR($$) {
 				my $oldentry = $oldbli->{$type}->{$gid}->{$bid};
 				my $olddata = tlv_encode($oldentry->{data});
 				$session->log_printf(OSCAR_DBG_DEBUG, "Old BLI entry %s 0x%04X/0x%04X/0x%04X with %d bytes of data:%s", $oldentry->{name}, $type, $gid, $bid, length($olddata), hexdump($olddata));
+				my $delete = 0;
 				if(exists($newbli->{$type}) and exists($newbli->{$type}->{$gid}) and exists($newbli->{$type}->{$gid}->{$bid})) {
 					my $newentry = $newbli->{$type}->{$gid}->{$bid};
 					my $newdata = tlv_encode($newentry->{data});
@@ -239,26 +246,43 @@ sub BLI_to_OSCAR($$) {
 						$newentry->{name} eq $oldentry->{name}
 					  and	$newdata eq $olddata;
 
-					$session->log_print(OSCAR_DBG_DEBUG, "Modifying.");
-					$modcount++;
+					# Apparently, we can't modify the name of a buddylist entry?
+					if($newentry->{name} ne $oldentry->{name}) {
+						$delete = 1;
+					} else {
+						$session->log_print(OSCAR_DBG_DEBUG, "Modifying.");
+						$modcount++;
 
-					$oscar->snac_put(family => 0x13, subtype => 0x9, reqdata => {desc => "modifying ".(BUDTYPES)[$type]." $newentry->{name}", type => $type, gid => $gid, bid => $bid}, data =>
-						pack("na* nnn na*",
-							length($newentry->{name}),
-							$newentry->{name},
-							$gid,
-							$bid,
-							$type,
-							length($newdata),
-							$newdata
-						)
-					);
+						push @snacqueue, $oscar->snac_encode(family => 0x13, subtype => 0x9, reqdata => {desc => "modifying ".(BUDTYPES)[$type]." $newentry->{name}", type => $type, gid => $gid, bid => $bid}, data =>
+							pack("na* nnn na*",
+								length($newentry->{name}),
+								$newentry->{name},
+								$gid,
+								$bid,
+								$type,
+								length($newdata),
+								$newdata
+							)
+						);
+					}
 				} else {
+					$delete = 1;
+				}
+
+				if($delete) {
 					$session->log_print(OSCAR_DBG_DEBUG, "Deleting.");
 					$modcount++;
 
-					$oscar->snac_put(family => 0x13, subtype => 0xA, reqdata => {desc => "deleting ".(BUDTYPES)[$type]." $oldentry->{name}", type => $type, gid => $gid, bid => $bid}, data => 
-						pack("nnnnn", 0, $gid, $bid, $type, 0)
+					push @snacqueue, $oscar->snac_encode(family => 0x13, subtype => 0xA, reqdata => {desc => "deleting ".(BUDTYPES)[$type]." $oldentry->{name}", type => $type, gid => $gid, bid => $bid}, data => 
+						pack("na* nnn na*",
+							length($oldentry->{name}),
+							$oldentry->{name},
+							$gid,
+							$bid,
+							$type,
+							length($olddata),
+							$olddata
+						)
 					);
 				}
 			}
@@ -269,14 +293,14 @@ sub BLI_to_OSCAR($$) {
 	foreach my $type(keys %$newbli) {
 		foreach my $gid(keys %{$newbli->{$type}}) {
 			foreach my $bid(keys %{$newbli->{$type}->{$gid}}) {
-				next if exists($oldbli->{$type}) and exists($oldbli->{$type}->{$gid}) and exists($oldbli->{$type}->{$gid}->{$bid});
+				next if exists($oldbli->{$type}) and exists($oldbli->{$type}->{$gid}) and exists($oldbli->{$type}->{$gid}->{$bid}) and $oldbli->{$type}->{$gid}->{$bid}->{name} eq $newbli->{$type}->{$gid}->{$bid}->{name};
 				my $entry = $newbli->{$type}->{$gid}->{$bid};
 				my $data = tlv_encode($entry->{data});
 
 				$session->log_printf(OSCAR_DBG_DEBUG, "New BLI entry %s 0x%04X/0x%04X/0x%04X with %d bytes of data:%s", $entry->{name}, $type, $gid, $bid, length($data), hexdump($data));
 				$modcount++;
 
-				$oscar->snac_put(family => 0x13, subtype => 0x8, reqdata => {desc => "adding ".(BUDTYPES)[$type]." $entry->{name}", type => $type, gid => $gid, bid => $bid}, data =>
+				push @snacqueue, $oscar->snac_encode(family => 0x13, subtype => 0x8, reqdata => {desc => "adding ".(BUDTYPES)[$type]." $entry->{name}", type => $type, gid => $gid, bid => $bid}, data =>
 					pack("na* nnn na*",
 						length($entry->{name}),
 						$entry->{name},
@@ -291,11 +315,15 @@ sub BLI_to_OSCAR($$) {
 		}
 	}
 
-	$oscar->snac_put(family => 0x13, subtype => 0x12); # End BL mods
+	push @snacqueue, $oscar->snac_encode(family => 0x13, subtype => 0x12); # End BL mods
 
 	$session->{blold} = $oldbli;
 	$session->{blinternal} = $newbli;
 
+	$oscar->flap_put(shift @snacqueue);
+
+	$session->{snacqueue} = \@snacqueue;
+
 	# OSCAR doesn't send an 0x13/0xE if we don't actually modify anything.
 	$session->callback_buddylist_ok() unless $modcount;
 
